/**
 * @file matrix_test.c
 * @author Florian Burgener (florian.burgener@etu.hesge.ch) et Quentin Fasler
 * @brief Seul les tests ont été fait en commun pour gagner du temps.
 * @version 1.0
 * @date 2021-11-16
 *
 * @copyright Copyright (c) 2021
 *
 */

#include "matrix.h"

#include <assert.h>
#include <math.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

int main() {
    // Init
    matrix m;
    int values[6] = {2, 1, -1, -2, 3, 1};
    int values2[6] = {2, 1, -1, -2, 3, 0};
    int values3[9] = {2, 1, -1, -2, 3, 0, 6, 8, 10};

    // Test : matrix_init
    matrix m0;
    assert(matrix_init(&m0, 2, 3, 1000) == ok);
    assert(m0.m == 2);
    assert(m0.n == 3);
    assert(m0.data[0][0] == 1000);
    assert(m0.data[0][1] == 1000);
    assert(m0.data[0][2] == 1000);
    assert(m0.data[1][0] == 1000);
    assert(m0.data[1][1] == 1000);
    assert(m0.data[1][2] == 1000);

    // Test : matrix_init_from_array
    matrix m1;
    assert(matrix_init_from_array(&m1, 2, 3, values, 6) == ok);
    assert(m1.m == 2);
    assert(m1.n == 3);
    assert(m1.data[0][0] == 2);
    assert(m1.data[0][1] == 1);
    assert(m1.data[0][2] == -1);
    assert(m1.data[1][0] == -2);
    assert(m1.data[1][1] == 3);
    assert(m1.data[1][2] == 1);

    // Init matrix m, m will never be changed.
    matrix_init_from_array(&m, 2, 3, values, 6);

    // Test : matrix_print
    assert(ok == matrix_print(m1));

    // Test : matrix_destroy
    assert(matrix_destroy(&m1) == ok);
    assert(m1.m == -1);
    assert(m1.n == -1);
    assert(m1.data == NULL);

    // Test : matrix_clone
    matrix m2;
    assert(matrix_clone(&m2, m) == ok);
    assert(m2.m == 2);
    assert(m2.n == 3);
    assert(m2.data[0][0] == 2);
    assert(m2.data[0][1] == 1);
    assert(m2.data[0][2] == -1);
    assert(m2.data[1][0] == -2);
    assert(m2.data[1][1] == 3);
    assert(m2.data[1][2] == 1);

    // Test : matrix_transpose
    matrix m3;
    assert(matrix_transpose(&m3, m) == ok);
    assert(m3.m == 3);
    assert(m3.n == 2);
    assert(m3.data[0][0] == 2);
    assert(m3.data[1][0] == 1);
    assert(m3.data[2][0] == -1);
    assert(m3.data[0][1] == -2);
    assert(m3.data[1][1] == 3);
    assert(m3.data[2][1] == 1);

    // Test : matrix_extract_submatrix
    matrix m4;
    assert(matrix_extract_submatrix(&m4, m, 0, 2, 0, 2) == ok);
    assert(m4.data[0][0] == 2);
    assert(m4.data[0][1] == 1);
    assert(m4.data[1][0] == -2);
    assert(m4.data[1][1] == 3);

    // Test : matrix_is_equal
    matrix m5_A;
    matrix_init_from_array(&m5_A, 2, 3, values, 6);
    matrix m5_B;
    matrix_init_from_array(&m5_B, 2, 3, values, 6);
    matrix m5_C;
    matrix_init_from_array(&m5_C, 2, 3, values2, 6);
    matrix m5_D;
    matrix_init_from_array(&m5_D, 3, 3, values3, 9);
    assert(matrix_is_equal(m5_A, m5_B) == true);
    assert(matrix_is_equal(m5_A, m5_C) == false);
    assert(matrix_is_equal(m5_A, m5_D) == false);

    // Test : matrix_get
    int32_t elem;
    matrix_get(&elem, m, 0, 1);
    assert(elem == -2);

    // Test : matrix_set
    matrix_set(m, 0, 1, 1000);
    assert(m.data[1][0] == 1000);

    matrix_destroy(&m0);
    // m1 is already destroyed.
    matrix_destroy(&m2);
    matrix_destroy(&m3);
    matrix_destroy(&m4);
    matrix_destroy(&m5_A);
    matrix_destroy(&m5_B);
    matrix_destroy(&m5_C);
    matrix_destroy(&m5_D);
    matrix_destroy(&m);
    return EXIT_SUCCESS;
}

